@@ -4,7 +4,7 @@
 #include "light.hpp"
 #include "py_common.hpp"
 
-class Scene {
+class scene {
 public:
     // must be thread-safe
     virtual color calculate_color(int x,int y,int w,int h) const = 0;
@@ -23,6 +23,7 @@ struct color_obj_base {
     static PyTypeObject *_pytype;
     static PyTypeObject *pytype() { return _pytype; }
 #endif
+    PyObject_HEAD
 };
 template<> struct _wrapped_type<color> {
     typedef simple_py_wrapper<color,color_obj_base> type;
